==> gtkpool_0.4.1.orig.tar.gz: gtkpool-0.4.1/gtkpool/sound.cpp <==
/***************************************************************************
                          sound.cpp  -  description
                             -------------------
    begin                : Tue Jan 8 2002
    copyright            : (C) 1999-2002 by Jacques Fortier & Brian Ashe
    email                : gtkpool@seul.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <errno.h>
#include "sound.h"

SoundError::SoundError(char *desc, int c, int r = 0, int g = 0){
	description = strdup(desc);
	code = c;
	requested = r;
	given = g;
}
SoundError::~SoundError(){
	free(description);
}

int audio_fd;
bool sound_enabled;

	// must add error checking!
	// must add error checking!!!
	// must add error checking!!!!!
	// must add error checking!!!!!!!
	// must add error checking!!!!!
	// must add error checking!!!
	// must add error checking!

void play_sound(char *sound, int count)
{
	if(!sound_enabled)
	{
		g_message("Sound not available.");
		return;
	}
	if(audio_fd < 0)
	{
		g_message("Sound device not open.");
		return;
	}
	if(write(audio_fd, sound, count) < 0)
	{
		g_message("Write to sound device failed.");
		return;
	}
	ioctl(audio_fd, SNDCTL_DSP_POST, 0);
}

int load_sound(char *file, char *buffer, int *count)
{
	int fd = open(file, O_RDONLY);
	*count = read(fd, buffer, *count);
	close(fd);
	return *count;
}

void open_sound_device(char *name)
{
	audio_fd = open(name, O_WRONLY);
	if(audio_fd < 0)
		throw SoundError(strerror(errno), SE_OPENFAIL);
}

void configure_sound_device()
{
	int fragment = 0x7fff0008;
	if( ioctl(audio_fd, SNDCTL_DSP_SETFRAGMENT, &fragment) < 0 )
		throw SoundError(strerror(errno), SE_FRAGFAIL, fragment);
	if((fragment & 0x0000ffff) != 8)
		throw SoundError("Sound card does not support appropriate fragment size", SE_FRAGFAIL, fragment);
	sound_enabled = true;
}

==> gtkpool_0.5.0.orig.tar.gz: gtkpool-0.5.0/gtkpool/sound.cpp <==
/***************************************************************************
                          sound.cpp  -  description
                             -------------------
    begin                : Tue Jan 8 2002
    copyright            : (C) 1999-2002 by Brian Ashe & Jacques Fortier
    email                : gtkpool@seul.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <errno.h>
#include "sound.h"

SoundError::SoundError(char *desc, int c, int r = 0, int g = 0){
	description = strdup(desc);
	code = c;
	requested = r;
	given = g;
}
SoundError::~SoundError(){
	free(description);
}

int audio_fd;
bool sound_enabled;

	// must add error checking!
	// must add error checking!!!
	// must add error checking!!!!!
	// must add error checking!!!!!!!
	// must add error checking!!!!!
	// must add error checking!!!
	// must add error checking!

void play_sound(char *sound, int count)
{
	//int fragment = 0x20008;
	if(!sound_enabled)
	{
		g_message("Sound not available.");
		return;
	}
	if(audio_fd < 0)
	{
		g_message("Sound device not open.");
		return;
	}
	
	if(write(audio_fd, sound, count) < 0)
	{
		g_message("Write to sound device failed.");
		return;
	}
	
	ioctl(audio_fd, SNDCTL_DSP_POST, 0);
}

int load_sound(char *file, char *buffer, int *count)
{
	int fd = open(file, O_RDONLY);
	*count = read(fd, buffer, *count);
	close(fd);
	return *count;
}

void open_sound_device(char *name)
{
	if ((audio_fd = open("/dev/dsp", O_WRONLY, 0)) == -1) {
		g_message("Could Not open Audio Device");
	}
}

void close_sound_device()
{
	 close (audio_fd);
}

void configure_sound_device()
{
	int fragment = 0x00020008;
	int format = AFMT_U8;
	int channels = 2;
	int speed = 44100;
	
	if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &format) == -1) {
		perror("SNDCTL_DSP_SETFMT");
	}
	if (ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &channels) == -1) {
		perror("SNDCTL_DSP_CHANNELS");
	}
	if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &speed) == -1) {
		perror("SNDCTL_DSP_SPEED");
	}
	if( ioctl(audio_fd, SNDCTL_DSP_SETFRAGMENT, &fragment) == -1 ) {
		perror("SNDCTL_DSP_SETFRAGMENT");
	}
	
	sound_enabled = true;
}
